"""GUI styles and themes."""

from .themes import (
    DARK_COLORS,
    LIGHT_COLORS,
    Theme,
    apply_current_theme,
    apply_theme,
    get_colors,
    get_current_theme,
    get_stylesheet,
    get_system_theme,
)

__all__ = [
    "Theme",
    "LIGHT_COLORS",
    "DARK_COLORS",
    "apply_current_theme",
    "apply_theme",
    "get_colors",
    "get_current_theme",
    "get_stylesheet",
    "get_system_theme",
]
